@@ -28,6 +28,7 @@
 /* Prototypes for static functions. */
 static void add_info_window_row(GtkWidget *table, gint pos,
 				gchar *text1, gchar *text2);
+void popup_songinfo_window(int song);
 
 /* Handle selections from the list. */
 void handle_list_select(GtkWidget *gtklist, gint row, gint column,
@@ -37,6 +38,21 @@ void handle_list_select(GtkWidget *gtklist, gint row, gint column,
   update_window();
 }
 
+/* Handle clicks on the status bar's event box. */
+void handle_eventbox_click(GtkWidget *widget, GdkEventButton *event,
+			   gpointer data) {
+  if (event->button == 3) {
+    /* This is a right button click. We want the context menu, so pass
+       it on to handle_menu_popup. */
+    handle_menu_popup(widget, event, data);
+
+  } else if (event->button == 1) {
+    /* This is a left button click. Pop up the song info window for
+       the current song. */
+    popup_songinfo_window(last_played);
+  }
+}
+
 /* Handle right clicks which bring up the popup menu. */
 void handle_menu_popup(GtkWidget *widget, GdkEventButton *event, 
 		       gpointer data) {
@@ -79,24 +95,17 @@ void handle_fileselector_cancel(GtkWidget *widget, gpointer data) {
 /* Handle the "OK" button in the Load Playlist file selector. */
 void handle_load_ok(GtkWidget *widget, gpointer data) {
   clear_playlist();
-  read_playlist(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselector)));
+  read_playlist(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselector)), 0);
   gtk_widget_destroy(fileselector);
   fileselector = NULL;
 }
 
 /* Handle the "OK" button in the Save Playlist file selector. */
 void handle_save_ok(GtkWidget *widget, gpointer data) {
-  FILE *f = fopen(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselector)), "w");
-  int i;
-
-  if (!f) {
-    status("Unable to save playlist.");
-    return;
-  }
-
-  for (i=0; i<listcount; i++) fprintf(f, "%s\n", get_songinfo(i)->name);
-  fclose(f);
+  gchar *n = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fileselector));
 
+  if (write_playlist(n)) status("Unable to save playlist.");
+  
   gtk_widget_destroy(fileselector);
   fileselector = NULL;
 }
@@ -217,13 +226,19 @@ static void add_info_window_row(GtkWidget *table, gint pos,
   gtk_widget_show(text);
 }
 
-/* Handle the items in the popup menu. */
+/* Handle "Info" in the popup menu. */
 void handle_menu_info(GtkWidget *widget, gpointer data) {
+  popup_songinfo_window(last_row_hit);
+}
+
+/* Pop up the song info window for a given song. */
+void popup_songinfo_window(int song) {
   songinfo *sinfo;
   GtkWidget *table;
   gchar buf[STRBUFSIZE], *p;
 
-  sinfo = get_songinfo(last_row_hit);
+  sinfo = get_songinfo(song);
+  if (!sinfo) return;
 
   /* If the window already exists, don't bother. */
   if (sinfo->info_window) return;
@@ -262,20 +277,8 @@ void handle_menu_down(GtkWidget *widget, gpointer data) {
 }
 
 void handle_menu_delete(GtkWidget *widget, gpointer data) {
-  /* If we're playing the removed song, stop; otherwise adjust the
-     current song number and all the previous links to match the list. */
-  if (last_row_hit == last_played) {
-    stop_playing();
-  } else if (last_played > last_row_hit) {
-    int i;
-
-    --last_played;
-    for (i = 0; i < listcount; i++) {
-      songinfo *s = get_songinfo(i);
-      if (s->previous > last_row_hit) --s->previous;
-    }
-  }
-  gtk_clist_remove(GTK_CLIST(filelist), last_row_hit);
+  if (last_row_hit == last_played) stop_playing();
+  delete_row(last_row_hit);
   update_window();
 }
 
@@ -373,17 +376,19 @@ void handle_menu_clear() {
 
 /* Handle the "delete_event" event. */
 gint handle_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data) {
-  /* Save the window position, if necessary. */
-  if (mode_save_window_pos) save_window_state();
-
   return(FALSE); /* Close the window, as FALSE */
 }
 
 /* Handle the "destroy" event. */
 void handle_destroy(GtkWidget *widget, gpointer data) {
+  /* Save the window position, if necessary. */
+  if (mode_save_window_pos) save_window_state();
+
   /* Stop the player. */
   stop_playing();
 
+  persist_playlist();
+
   /* Then quit the interface. */
   gtk_main_quit();
 }
