@@ -26,8 +26,9 @@
 #include "xhippo.h"
 
 /* Prototypes for static functions. */
-static void add_file_action(gchar *extension, gchar *action);
+static void add_file_action(gchar *extension, gchar *action, gchar *options);
 static void add_user_command(gchar *description, gchar *command);
+static void conf_read_playlist(gchar *name);
 
 /* Globals. */
 gboolean mode_play_on_start = 0, 
@@ -49,6 +50,10 @@ gboolean mode_play_on_start = 0,
   mode_write_playing = 0,
   mode_commandline_songs = 0,
   mode_commandline_dirs = 0,
+  mode_delete_when_played = 0,
+  mode_persist_playlist = 0,
+  mode_commandline_guess = 0,
+  mode_persist_frequently = 0,
   attribute_mini = 0;
 guint mode_skip_path = 0;
 gchar *attribute_playlist_dir = NULL;
@@ -81,22 +86,43 @@ static configcommand configcommands[] = {
   { 0, CT_BOOL, "writeplaying", &mode_write_playing },
   { 0, CT_BOOL, "commandlinesongs", &mode_commandline_songs },
   { 0, CT_BOOL, "commandlinedirs", &mode_commandline_dirs },
+  { 0, CT_BOOL, "deletewhenplayed", &mode_delete_when_played },
+  { 0, CT_BOOL, "persistplaylist", &mode_persist_playlist },
+  { 0, CT_BOOL, "commandlineguess", &mode_commandline_guess },
+  { 0, CT_BOOL, "persistfrequently", &mode_persist_frequently },
   { 0, CT_INT, "skippath", &mode_skip_path },
   { 0, CT_STRING, "playlistdir", &attribute_playlist_dir },
   { 0, CT_SORTTYPE, "sortonload", &attribute_sort_on_load },
-  { 0, CT_FUNCTION2, "type", add_file_action },
+  { 0, CT_FUNCTION23, "type", add_file_action },
   { 0, CT_FUNCTION2, "usercommand", add_user_command },
-  { 1, CT_FUNCTION1, "load", read_playlist },
+  { 1, CT_FUNCTION1, "load", conf_read_playlist },
   { 1, CT_FUNCTION1, "exec", system },
   { 0, CT_END, NULL, NULL }
 };
 
+/* Read a playlist. */
+static void conf_read_playlist(gchar *name) {
+  read_playlist(name, 0);
+}
+
 /* Add a file action to the list. */
-static void add_file_action(gchar *extension, gchar *action) {
+static void add_file_action(gchar *extension, gchar *action, gchar *options) {
+  char *p;
   fileaction *fa = (fileaction *)malloc(sizeof(fileaction));
 
   fa->extension = strdup(extension);
   fa->action = strdup(action);
+
+  /* Make fa->name contain the name up to the first space. */
+  fa->name = strdup(action);
+  p = strchr(fa->name, ' ');
+  if (p) *p = '\0';
+
+  fa->setgroup = fa->nullinput = 0;
+  if (options) {
+    if (strchr(options, 'g')) fa->setgroup = 1;
+    if (strchr(options, 'i')) fa->nullinput = 1;
+  }
   
   fa->next = fileactions;
   fileactions = fa;
@@ -124,7 +150,7 @@ static void add_user_command(gchar *description, gchar *command) {
    reading the config file in the future (perhaps for remote
    control). */
 void execute_command(gchar *command, gint pass) {
-  gchar *cmd = strdup(command), *opta, *optb = NULL;
+  gchar *cmd = strdup(command), *opta, *optb = NULL, *optc = NULL;
   configcommand *cc;
 
   if (!*cmd) return;
@@ -134,7 +160,11 @@ void execute_command(gchar *command, gint pass) {
   if (opta) {
     *opta++ = '\0';
     optb = strchr(opta, ':');
-    if (optb) *optb++ = '\0';
+    if (optb) {
+      *optb++ = '\0';
+      optc = strchr(optb, ':');
+      if (optc) *optc++ = '\0';
+    }
   }
 
   for (cc = configcommands; cc->type != CT_END; cc++) {
@@ -209,6 +239,16 @@ void execute_command(gchar *command, gint pass) {
       func(opta, optb);
       break;
     }
+    case CT_FUNCTION23: {
+      void (*func)(gchar *a, gchar *b, gchar *c) = cc->target;
+
+      if (!opta || !optb) {
+	fprintf(stderr, "Command '%s' needs at least two arguments.", cmd);
+	goto out;
+      }
+      func(opta, optb, optc);
+      break;
+    }
     case CT_END:
       /* This will not happen. */
       break;
