@@ -27,7 +27,6 @@
 
 /* Prototypes for static functions. */
 static const char *find_swap_point(const char *a);
-static fileaction *get_action(const gchar *name);
 void sort_as_appropriate(void);
 #ifdef HAVE_LIBID3
 static int get_id3_field(ID3Tag *tag, ID3_FrameID frameid, char *buf, int len);
@@ -36,35 +35,41 @@ static int get_id3_field(ID3Tag *tag, ID3_FrameID frameid, char *buf, int len);
 static int add_directory_callback(const char *name, const struct stat *sb,
 				  int flag, struct FTW *ftw);
 #endif
+static void kill_child(int signal);
 
 /* Globals. */
-pid_t childpid;
+pid_t childpid = 0;
 guint contextid, 
   listcount = 0, 
   playing = 0, 
-  paused = 0, 
-  donotstopflag;
+  paused = 0;
 gint last_played = -1,
   last_row_hit;
-gchar *last_player = NULL;
+fileaction *last_action = NULL;
+
+/* Send a signal to the child process (or process group). */
+static void kill_child(int signal) {
+  kill(last_action->setgroup ? -childpid : childpid, signal);
+}
 
 /* Stop playing. */
 void stop_playing() {
   if (!playing) return;
+  playing = 0;
+
   if (!childpid) return;
 
   /* Wake up a paused player. */
   if (paused) {
-    kill(childpid, SIGCONT);
+    kill_child(SIGCONT);
     paused = 0;
   }
 
-  playing = 0;
-  kill(childpid, SIGTERM);
+  kill_child(SIGTERM);
 
-  /* Pause (to avoid the formation of zombies) until the child
-     dies. childpid is reset in the SIGCHLD handler. */
-  while (childpid) pause();
+  /* Wait for the child to exit. */
+  waitpid(childpid, NULL, 0);
+  childpid = 0;
 }
 
 /* Toggle paused status. */
@@ -73,17 +78,17 @@ void pause_playing() {
   if (!childpid) return;
 
   if (paused) {
-    kill(childpid, SIGCONT);
+    kill_child(SIGCONT);
     paused = 0;
   } else {
-    kill(childpid, SIGSTOP);
+    kill_child(SIGSTOP);
     paused = 1;
   }
 }
 
 /* Get the action for a given filename. Return NULL if a matching
    action isn't found. */
-static fileaction *get_action(const gchar *name) {
+fileaction *get_action(const gchar *name) {
   const gchar *type;
 
   type = name + strlen(name);
@@ -104,36 +109,45 @@ static fileaction *get_action(const gchar *name) {
 
 /* Start playing song number num. Stop the old one first. */
 void start_playing(gint num) {
-  gchar *name, *p;
-  fileaction *fa;
+  gchar *name, c;
   int argcount, i;
+  int pipefd[2];
 
   if (num < 0 || num >= listcount) return;
 
-  last_played = num;
+  if (childpid) stop_playing();
 
-  /* Stop the previous song. */
-  if (donotstopflag) {
-    donotstopflag = 0;
-  } else {
-    stop_playing();
-  }
+  last_played = num;
 
   /* Get the extension of the filename. */
   name = get_songinfo(num)->name;
-  fa = get_action(name);
-  if (!fa) {
+  last_action = get_action(name);
+  if (!last_action) {
     status(_("Unable to guess type."));
     return;
   }
   
+  /* Create a pipe to synchronise with the child. */
+  if (pipe(pipefd) < 0) return;
+
   /* Fork off a child process to run the player in. */
+  playing = 1;
+  paused = 0;
   childpid = fork();
+  if (childpid < 0) return;
+
   if (!childpid) {
     gchar *command, **argptr, *p;
 
+    /* Before we do anything else, synchonise with the main process;
+       if the player dies immediately it's possible for xhippo's
+       sigchld handler to get called before the child's pid has even
+       made it into childpid, which makes for interesting
+       deadlocks. */
+    read(pipefd[0], &c, 1);
+
     /* Build the argptr array for execvp. */
-    command = strdup(fa->action);
+    command = strdup(last_action->action);
     argcount = 2;
     for (p = command; p; p = strchr(p, ' ')) {
       argcount++;
@@ -149,15 +163,22 @@ void start_playing(gint num) {
     argptr[i++] = NULL;
 
     /* Redirect stdout and stderr to /dev/null if necessary. */
-    if (mode_hide_player_output) {
+    if (mode_hide_player_output || last_action->nullinput) {
       int fd;
 
       fd = open("/dev/null", O_RDWR);
-      dup2(fd, 1); /* stdout */
-      dup2(fd, 2); /* stderr */
+      if (last_action->nullinput) dup2(fd, 0); /* stdin */
+      if (mode_hide_player_output) {
+	dup2(fd, 1); /* stdout */
+	dup2(fd, 2); /* stderr */
+      }
       close(fd);
     }
 
+    /* Set process group so that all processes in the group get
+       killed. */
+    if (last_action->setgroup) setpgid(0, 0);
+
     /* At this point we've still got command and argptr allocated. */
     execvp(argptr[0], argptr);
     fprintf(stderr, _("Unable to start player \"%s\".\n"), argptr[0]);
@@ -165,14 +186,10 @@ void start_playing(gint num) {
     _exit(20);
   }
 
-  /* Update last_player. */
-  if (last_player) free(last_player);
-  last_player = strdup(fa->action);
-  p = strchr(last_player, ' ');
-  if (p) *p = '\0';
-
-  playing = 1;
-  paused = 0;
+  /* Synchronise with the child. */
+  write(pipefd[1], "!", 1);
+  close(pipefd[0]);
+  close(pipefd[1]);
 
   if (mode_write_playing) {
     gchar csname[STRBUFSIZE];
@@ -187,6 +204,8 @@ void start_playing(gint num) {
       fclose(f);
     }
   }
+
+  if (mode_persist_frequently) persist_playlist();
 }
 
 /* Pick and play the previous song. */
@@ -223,7 +242,7 @@ void pick_next() {
       if (newitem == listcount) {
 	newitem = 0;
         if (mode_one_time) {
-          playing = 0;
+          stop_playing();
           return;
         }
       }
@@ -239,7 +258,8 @@ void pick_next() {
 	  if (newitem==first) {
 	    for (i=0; i<listcount; i++) get_songinfo(i)->played = 0;
             if (mode_one_time) {
-              playing = 0;
+              stop_playing();
+	      status(_("All items played."));
               return;
             }
 	  }
@@ -348,12 +368,21 @@ void read_song_info(songinfo *sinfo, struct stat *st) {
 }
 
 /* Called to add a new file to the playlist. */
-void add_file(const gchar *name) {
+void add_file(const gchar *initname) {
+  gchar *name;
   const gchar *basename;
   gchar *dispname, *tag;
   songinfo *sinfo;
   struct stat st;
 
+  /* Prepend the cwd if the name doesn't start with a slash. */
+  if (initname[0] == '/') {
+    name = strdup(initname);
+  } else {
+    name = malloc(strlen(startup_cwd) + strlen(initname) + 1);
+    sprintf(name, "%s%s", startup_cwd, initname);
+  }
+
   if (!mode_no_check_files) {
     /* Note: this uses stat rather than filetype() because we need the
        other information from the stat structure and this avoids
@@ -417,7 +446,7 @@ void add_file(const gchar *name) {
 
   /* Build a songinfo struct. */
   sinfo = malloc(sizeof(songinfo));
-  sinfo->name = strdup(name);
+  sinfo->name = name;
   sinfo->display_name = dispname;
   sinfo->played = 0;
   sinfo->inforead = 0;
@@ -438,10 +467,12 @@ static int add_directory_callback(const char *name, const struct stat *sb,
   if (flag == FTW_F && get_action(name)) add_file(name);
   return 0;
 }
+#endif
 
 /* Called to add all the playable files in a directory to a
    playlist. */
 void add_directory(const gchar *name) {
+#ifdef HAVE_NFTW
   gtk_clist_freeze(GTK_CLIST(filelist));
 
   /* We use an arbitrary maximum depth of 100, since nftw (at least in
@@ -452,8 +483,10 @@ void add_directory(const gchar *name) {
 
   sort_as_appropriate();
   gtk_clist_thaw(GTK_CLIST(filelist));
-}
+#else
+  /* We don't have nftw; just don't do anything (for now). */
 #endif
+}
 
 /* Clear the playlist. */
 void clear_playlist() {
@@ -538,8 +571,20 @@ void sortplaylist(int (*func)(const void *a, const void *b),
   free(sl);
 }
 
-/* Read a playlist. */
-void read_playlist(gchar *name) {
+/* Write a playlist. Returns 1 on error, 0 on success. */
+int write_playlist(gchar *name) {
+  int i;
+  FILE *f = fopen(name, "w");
+
+  if (!f) return 1;
+  for (i=0; i<listcount; i++) fprintf(f, "%s\n", get_songinfo(i)->name);
+  fclose(f);
+  return 0;
+}
+
+/* Read a playlist. If use_cwd is 1, use the cwd as the dirname rather
+   than the dirname of the playlist. */
+void read_playlist(gchar *name, int use_cwd) {
   FILE *f;
   char line[STRBUFSIZE], fullname[STRBUFSIZE], dirname[STRBUFSIZE], 
     *slash, *filename;
@@ -555,7 +600,9 @@ void read_playlist(gchar *name) {
 
   /* Work out the dirname if we've got one. */
   slash = strrchr(name, '/');
-  if (slash) {
+  if (use_cwd) {
+    snprintf(dirname, STRBUFSIZE, "%s", startup_cwd);
+  } else if (slash) {
     *slash = '\0';
     snprintf(dirname, STRBUFSIZE, "%s/", name);
     *slash = '/';
@@ -576,7 +623,7 @@ void read_playlist(gchar *name) {
     
     if (*line == '!') {
       /* Allow inclusion of a playlist from a playlist. */
-      read_playlist(fullname);
+      read_playlist(fullname, use_cwd);
     } else {
       add_file(fullname);
     }
@@ -637,6 +684,24 @@ void destroy_songinfo(gpointer sinfo) {
   --listcount;
 }
 
+/* Delete a row from the list. */
+void delete_row(gint location) {
+  int i;
+
+  /* Adjust the current song number. We decrement this even if we just
+     played the deleted song, because that makes non-random mode do
+     sensible things when moving to the next song. */
+  if (last_played >= location) --last_played;
+  
+  /* Adjust all the previous links to match the list. */
+  for (i = 0; i < listcount; i++) {
+    songinfo *s = get_songinfo(i);
+    if (s->previous > location) --s->previous;
+  }
+
+  gtk_clist_remove(GTK_CLIST(filelist), location);
+}
+
 /* Insert a row into the list based on a filled-in songinfo structure. 
    Call as insert_row(X, listcount); to append to the end. */
 void insert_row(songinfo *sinfo, gint location) {
@@ -666,3 +731,11 @@ void move_row(gint src, gint dest) {
   gtk_clist_thaw(GTK_CLIST(filelist));
 }
 
+/* Save the persistent playlist. */
+void persist_playlist() {
+  gchar buf[STRBUFSIZE];
+  
+  /* Save the playlist. */
+  snprintf(buf, STRBUFSIZE, "%s/.xhippo/saved_playlist", getenv("HOME"));
+  if (mode_persist_playlist) write_playlist(buf);
+}
