@@ -42,6 +42,8 @@ GtkWidget *window, *restartbutton, *stopbutton, *buttons_box,
   *up_item, *down_item, *delete_item, *info_item, *eventbox;
 GtkAdjustment *vadj;
 GtkAccelGroup *accel_group;
+int sigchld_pipe[2];
+gchar startup_cwd[STRBUFSIZE];
 
 /* Show a message on the status bar */
 void status(char *message) {
@@ -65,10 +67,11 @@ void update_window() {
     } else {
       /* Put the "Playing with" string in the status bar. */
       if (mode_show_pid) {
-	snprintf(statbuf, STRBUFSIZE, _("Playing with %s (pid %d)"), last_player,
-		 childpid);
+	snprintf(statbuf, STRBUFSIZE, _("Playing with %s (pid %d)"),
+		 last_action->name, childpid);
       } else {
-	snprintf(statbuf, STRBUFSIZE, _("Playing with %s."), last_player);
+	snprintf(statbuf, STRBUFSIZE, _("Playing with %s."),
+		 last_action->name);
       }
     }
 
@@ -86,27 +89,29 @@ void update_window() {
 
 /* A timeout function to be called after we've had a SIGCHLD. This
    does stuff that we can't do in the signal handler. */
-gint sigchld_timeout(gpointer data) {
-  donotstopflag = 1;
-  pick_next();
-  if (!playing) { /* reached end of one time list? */
-    donotstopflag = 1;
-    status(_("All items played."));
+void sigchld_backend(gpointer data, gint source, GdkInputCondition cond) {
+  gchar c;
+  pid_t deadpid;
+
+  /* Read the character off the pipe. */
+  read(sigchld_pipe[0], &c, 1);
+
+  /* If the child that has exited isn't the player process, we're not
+     interested. */
+  deadpid = waitpid(-1, NULL, WNOHANG);
+  if (deadpid < 1 || deadpid != childpid) return;
+  childpid = 0;
+
+  if (playing) {
+    if (mode_delete_when_played) delete_row(last_played);
+    pick_next();
   }
-  return FALSE; /* don't do the timeout again */
 }
 
 /* Called when the child process quits. */
 void handle_sigchld(int dummy) {
-  /* If the child that has exited isn't the player process, we're not
-     interested. */
-  if (waitpid(childpid, NULL, WNOHANG) == 0) return;
-
-  childpid = 0;
-
-  /* If we're in play mode, get the timeout called next time we hit
-     the main loop. */
-  if (playing) gtk_timeout_add(0, sigchld_timeout, NULL);
+  /* Leave it to GDK to find out about. */
+  write(sigchld_pipe[1], "!", 1);
 }
 
 #ifdef USEGNOMEMENUS
@@ -205,16 +210,28 @@ int main(int argc, char **argv) {
   gchar buf[STRBUFSIZE], wintitlestring[STRBUFSIZE];
   int c, j; /* for getopt */
 
+  /* Set up gettext. */
+  setlocale(LC_ALL, "");
+  bindtextdomain(PACKAGE, LOCALEDIR);
+  textdomain(PACKAGE);
+
   /* Check that HOME is available. */
   if (!getenv("HOME")) {
     fprintf(stderr, _("You must set $HOME to your home directory.\n"));
     exit(20);
   }
 
-  /* Set up gettext. */
-  setlocale(LC_ALL, "");
-  bindtextdomain(PACKAGE, LOCALEDIR);
-  textdomain(PACKAGE);
+  /* Get the CWD. */
+  if (getcwd(startup_cwd, STRBUFSIZE - 1)) {
+    /* We left 1 space above for the /. */
+    strcat(startup_cwd, "/");
+  } else {
+    /* Leave it blank; prepending it to paths won't do any damage then. */
+    *startup_cwd = '\0';
+  }
+
+  /* Set up the pipe used for the SIGCHLD handler. */
+  pipe(sigchld_pipe);
 
   /* Seed RNG. */
   srand(time(0));
@@ -288,7 +305,7 @@ int main(int argc, char **argv) {
   /* Create the event box for the status bar. */
   eventbox = gtk_event_box_new();
   gtk_signal_connect(GTK_OBJECT(eventbox), "button_press_event",
-		     GTK_SIGNAL_FUNC(handle_menu_popup), NULL);
+		     GTK_SIGNAL_FUNC(handle_eventbox_click), NULL);
   gtk_box_pack_start(GTK_BOX(all_box), eventbox, FALSE, FALSE, 0);
 
   /* Create the status bar. */
@@ -412,7 +429,7 @@ int main(int argc, char **argv) {
 
   /* Parse arguments. */
   while (1) {
-    char *argstring = "aslmpiwtVhoqOd1TbcWk:fD";
+    char *argstring = "aslmpiwtVhoqOd1TbcWk:fDxPg";
 #ifndef NOGETOPTLONG
     static struct option long_options[] = {
       {"autostart",0,0,0}, /* -a */
@@ -437,6 +454,9 @@ int main(int argc, char **argv) {
       {"skippath", required_argument, 0, 0}, /* -k */
       {"commandlinesongs",0,0,0}, /* -f */
       {"commandlinedirs",0,0,0}, /* -D */
+      {"deletewhenplayed",0,0,0}, /* -x */
+      {"persistplaylist",0,0,0}, /* -P */
+      {"commandlineguess",0,0,0}, /* -g */
       {0,0,0,0}
     };
     j = 0;
@@ -522,6 +542,15 @@ int main(int argc, char **argv) {
     case 'D': /* -d: directories (rather than playlists) on command line */
       mode_commandline_dirs = 1;
       break;
+    case 'x': /* -x: delete when played */
+      mode_delete_when_played = 1;
+      break;
+    case 'P': /* -P: persist playlist */
+      mode_persist_playlist = 1;
+      break;
+    case 'g': /* -g: guess command line arguments */
+      mode_commandline_guess = 1;
+      break;
     case 'V':
       print_version();
       exit(0);
@@ -571,6 +600,13 @@ int main(int argc, char **argv) {
 "                   Take the non-option arguments as directories to search\n"
 "                   for files to add to the playlist rather than playlists\n"
 "                   to load.\n"
+"-x  --deletewhenplayed\n"
+"                   Delete played songs from the playlist.\n"
+"-P  --persistplaylist\n"
+"                   Automatically load and save the current playlist.\n"
+"-g  --commandlineguess\n"
+"                   Attempt to guess based on the config settings whether\n"
+"                   non-option arguments are songs, dirs or playlists.\n"
 "-h  --help         This help.\n"
 "\n"
 "Report bugs to bug-xhippo@gnu.org.\n");
@@ -609,14 +645,30 @@ int main(int argc, char **argv) {
     gtk_box_pack_start(GTK_BOX(list_box), scrollbar, FALSE, FALSE, 0); 
   } 
 
+  /* Load the playlist if no other args are given. */
+  snprintf(buf, STRBUFSIZE, "%s/.xhippo/saved_playlist", getenv("HOME"));
+  if (mode_persist_playlist && optind >= argc) read_playlist(buf, 1);
+
   /* Deal with non-options */
   while (optind < argc) {
+    char *name = argv[optind++];
     if (mode_commandline_songs) {
-      add_file(argv[optind++]);
+      add_file(name);
     } else if (mode_commandline_dirs) {
-      add_directory(argv[optind++]);
+      add_directory(name);
     } else {
-      read_playlist(argv[optind++]);
+      if (mode_commandline_guess) {
+	fileaction *fa = get_action(name);
+	if (fa) {
+	  add_file(name);
+	} else if (filetype(name) == 2) {
+	  add_directory(name);
+	} else {
+	  read_playlist(name, 0);
+	}
+      } else {
+	read_playlist(name, 0);
+      }
     }
   }
   
@@ -664,9 +716,15 @@ int main(int argc, char **argv) {
   /* If mode_play_on_start, start playing. */
   if (mode_play_on_start) pick_next();
 
+  /* Make GDK monitor the pipe. */
+  gdk_input_add(sigchld_pipe[0], GDK_INPUT_READ, sigchld_backend, NULL);
+
   /* Main loop. */
   gtk_main();
 
+  /* Note that the interface doesn't exist any more, so you can't do
+     anything interesting here. */
+
   /* And exit cleanly. */
   exit(0);
 }
