@@ -77,6 +77,11 @@
 struct _fileaction {
   gchar *extension;        /* The extension of the file. */
   gchar *action;           /* The command to run. */
+  gchar *name;             /* The name of the command. */
+  char setgroup;           /* Whether the player should be run in a
+			      separate process group. */
+  char nullinput;          /* Whether the player's stdin should be
+                              connected to /dev/null. */
 
   struct _fileaction *next;
 };
@@ -115,7 +120,9 @@ enum _configtype {
   CT_STRING,                /* gchar * */
   CT_SORTTYPE,              /* sorttype */
   CT_FUNCTION1,             /* void function(gchar *arg) */
-  CT_FUNCTION2              /* void function(gchar *arg1, gchar *arg2) */
+  CT_FUNCTION2,             /* void function(gchar *arg1, gchar *arg2) */
+  CT_FUNCTION23             /* void function(gchar *arg1, gchar *arg2,
+			       gchar *arg3) (arg3 can be NULL) */
 };
 typedef enum _configtype configtype;
 
@@ -147,17 +154,18 @@ extern GtkWidget *window, *restartbutton, *stopbutton, *buttons_box,
   *up_item, *down_item, *delete_item, *info_item, *eventbox;
 extern GtkAdjustment *vadj;
 extern GtkAccelGroup *accel_group;
+extern int sigchld_pipe[2];
+extern gchar startup_cwd[STRBUFSIZE];
 
 /* Status variables */
 extern pid_t childpid;
 extern guint contextid, 
   listcount, 
   playing, 
-  paused, 
-  donotstopflag;
+  paused;
 extern gint last_played,
   last_row_hit;
-extern gchar *last_player;
+fileaction *last_action;
 
 /* Configuration options */
 extern gboolean mode_play_on_start, 
@@ -179,6 +187,10 @@ extern gboolean mode_play_on_start,
   mode_write_playing,
   mode_commandline_songs,
   mode_commandline_dirs,
+  mode_delete_when_played,
+  mode_persist_playlist,
+  mode_commandline_guess,
+  mode_persist_frequently,
   attribute_mini;
 extern guint mode_skip_path;
 extern gchar *attribute_playlist_dir;
@@ -194,24 +206,25 @@ extern usercommand *usercommands;
 
 songinfo *copy_songinfo(songinfo *src);
 void destroy_songinfo(gpointer sinfo);
+void delete_row(gint location);
 void insert_row(songinfo *sinfo, gint location);
 songinfo *get_songinfo(gint number);
 void move_row(gint src, gint dest);
+void persist_playlist(void);
 gint filetype(char *file);
 void chomp(gchar *line);
 void status(char *message);
 void update_window(void);
 void stop_playing(void);
 void pause_playing(void);
+fileaction *get_action(const gchar *name);
 void start_playing(gint num);
 void pick_prev(void);
 void pick_next(void);
 gchar *read_mp3_tag (const char *filename );
 void read_song_info(songinfo *sinfo, struct stat *st);
 void add_file(const gchar *name);
-#ifdef HAVE_NFTW
 void add_directory(const gchar *name);
-#endif
 void save_window_state(void);
 void read_window_state(void);
 void clear_playlist(void);
@@ -220,12 +233,15 @@ int sort_compare_swapped_name(const void *a, const void *b);
 int sort_compare_mtime(const void *a, const void *b);
 void sortplaylist(int (*func)(const void *a, const void *b),
 		  char needinfo);
-void read_playlist(gchar *name);
+void read_playlist(gchar *name, int use_cwd);
+int write_playlist(gchar *name);
 void execute_command(gchar *command, gint pass);
 void read_config(gchar *name, gint pass);
 void read_config_files(gint pass);
 void handle_list_select(GtkWidget *gtklist, gint row, gint column,
 			GdkEventButton *event, gpointer data);
+void handle_eventbox_click(GtkWidget *widget, GdkEventButton *event,
+			   gpointer data);
 void handle_menu_popup(GtkWidget *widget, GdkEventButton *event, 
 		       gpointer data);
 void handle_fileselector_cancel(GtkWidget *widget, gpointer data);
@@ -265,7 +281,7 @@ void handle_menu_clear(void);
 gint handle_delete_event(GtkWidget *widget, GdkEvent *event, gpointer data);
 void handle_destroy(GtkWidget *widget, gpointer data);
 void handle_configure(GtkWidget *widget, gpointer data);
-gint sigchld_timeout(gpointer data);
+void sigchld_backend(gpointer data, gint source, GdkInputCondition cond);
 void handle_sigchld(int dummy);
 #ifdef USEGNOMEMENUS
 void handle_about(GtkWidget *widget, void *data);
